﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Foundation.Messages;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;

namespace WorkflowScript_AfterLogisticUnitItemEnteredHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_AfterLogisticUnitItemEnteredHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<int?> ContainerDocEntry;
        public ReadWriteBinder<LogisticUnitItemGoodReceipt> LogUnitItem;
       


        // Output parameters * do not change *
        public ReadWriteBinder<bool> BackRequested;
        // Sub-flows
        // <none>

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_AfterLogisticUnitItemEnteredHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_AfterLogisticUnitItemEnteredHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            Message msg = null;
            string initialErrorKey = null;
            PmxItemInfo itemInfo = null;
            //ItemPrice itemPrice = null;
            //double price = 0;
            //string currency = null;
            //ProductInfo productInfo = null;

            IBarcodeController barcodeController = null;
            LogisticUnitItemGoodReceipt logUnitItem = LogUnitItem.Get();


            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
              delegate (PmxDbConnection conn, object[] parameters)
              {
                  PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
                  barcodeController = bcProv.GetBarcodeController();

                  return null;
              });

           

            Step_EnterUnitPrice:

            #region ENTER UNIT PRICE

            if (ContainerDocEntry.Get().HasValue || logUnitItem.PurchaseDocRef != null)
            {
                BackRequested.Set(false);
                return;
            }

            string localCurrency = null;
            int priceDecimals=2;

            sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
             delegate (PmxDbConnection conn, object[] parameters)
             {
                 SboCompanyProvider companyProv = new SboCompanyProvider((SboConnection) conn);
                 localCurrency = companyProv.GetAdminInfo().LocalCurrency;
                 priceDecimals = companyProv.GetAdminInfo().PriceDecimals;
                 return null;
             });


         
         
            itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), logUnitItem.ItemCode);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "TitleKey", "MSG_TITLE_ENTER_UNIT_PRICE",
                                "BarcodeController", barcodeController,
                                "NumberOfDigitsAfterDecimal", priceDecimals,
                                "StillToQuantity", logUnitItem.Quantity,
                                "MaximumQuantity", -1,
                               // "UOM", localCurrency,
                                "ItemDescription", itemInfo.CustomItemDescription,
                                "ItemCode", itemInfo.ItemCode,
                                "ItemBarcode", itemInfo.CodeBars,
                                "BatchNumber", logUnitItem.BatchNumber1,
                                "BestBeforeDate", logUnitItem.BestBeforeDate,
                                "InnerUOM", string.Empty , 
                                "UomToDisplayOnQtyProvider", localCurrency
                               ));

            msg = WaitForMessage();

            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                logUnitItem.UnitPrice  = ExtractParameter<double>(msg.Parameters, "quantity");
                LogUnitItem.Set(logUnitItem);
                BackRequested.Set(false);
                return; 

            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {

                BackRequested.Set(true);
                return;
            }

            #endregion

        }

#endregion

        #region Private methods


        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfo((string)parameters[0]);
        }


        #endregion


    }
}
